//----------------------------------------------------------------------
//
//  FILE:   PlotCollator.cpp
//  AUTHOR: Ian Ross
//  DATE:   29-MAR-2005
//
//----------------------------------------------------------------------
//
//  Plot collation class for MOTIF benchmarking plot document generator.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <iostream>
#include <fstream>
#include <unistd.h>

using namespace std;


// Local headers.

#include "PlotCollator.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

PlotCollator::PlotCollator(string outfile, string model, string title) :
  _outfile(outfile), _model(model), _title(title),
  _curr_var(0), _curr_setup(0)
{ }


PlotCollator::~PlotCollator()
{
  for (vector<VariableEntry*>::iterator vit = _vars.begin();
       vit != _vars.end(); ++vit) {
    VariableEntry *ventry = *vit;

    for (vector<SetupEntry*>::iterator sit = ventry->setups.begin();
         sit != ventry->setups.end(); ++sit) {
      SetupEntry *sentry = *sit;
      
      for (vector<PlotEntry*>::iterator plit = sentry->plots.begin();
           plit != sentry->plots.end(); ++plit) {
        PlotEntry *plentry = *plit;
        delete plentry;
      }
      sentry->plots.clear();
      delete sentry;
    }
    ventry->setups.clear();
    delete ventry;
  }
  _vars.clear();
}


void PlotCollator::new_variable(string var_name)
{
  cout << "New variable: " << var_name << endl;
  _curr_var = new VariableEntry(var_name);
  _vars.push_back(_curr_var);
}


void PlotCollator::new_setup(const PlotSetup &setup)
{
  cout << "  New setup: " << setup << endl;
  _curr_setup = new SetupEntry(setup);
  _curr_var->setups.push_back(_curr_setup);
}


void PlotCollator::add_plot(const BenchmarkPlot *plot, float angle)
{
  _curr_setup->plots.push_back(new PlotEntry(plot, angle));
}


void PlotCollator::operator()(void)
{
  string latex_file = TempFile::get("tex");
  ofstream lstr(latex_file.c_str());

  lstr << "\\documentclass[12pt]{article}" << endl;
  lstr << "\\usepackage{vmargin,graphicx}" << endl;
  lstr << "\\renewcommand{\\familydefault}{cmss}" << endl;
  lstr << "\\setpapersize{A4}" << endl;
  lstr << "\\setmarginsrb{0.5in}{0.5in}{0.5in}{0.25in}"
    "{0pt}{0pt}{0cm}{2cm}" << endl;
  lstr << "\\pagestyle{plain}" << endl;
  lstr << "\\begin{document}" << endl;
  lstr << "\\setlength{\\parindent}{0pt}" << endl;
  lstr << "\\setlength{\\parskip}{0.5cm}" << endl;
  lstr << "\\thispagestyle{plain}" << endl;
  lstr << "\\begin{center}" << endl;
  lstr << "{\\LARGE " << _model << " " << _title << "}" << endl;
  lstr << "\\end{center}" << endl;
  lstr << "\\vspace{0.5cm}" << endl;
  lstr << "\\makeatletter" << endl;

  int setup_count = 1;
  for (vector<VariableEntry*>::iterator vit = _vars.begin();
       vit != _vars.end(); ++vit) {
    VariableEntry *ventry = *vit;

    if (ventry->setups.size() == 0) continue;

    // Write variable line.
    lstr << "{\\Large " << ventry->var_name << "}" << endl;

    for (vector<SetupEntry*>::iterator sit = ventry->setups.begin();
         sit != ventry->setups.end(); ++sit) {
      SetupEntry *sentry = *sit;

      // Write setup line.
      string setup_string = string(sentry->setup);
      string::size_type dash = setup_string.find('-');
      if (dash != string::npos) setup_string.replace(dash, 1, "$-$");
      lstr << "\\@dottedtocline{1}{1cm}{1cm}{"
           << setup_string << "}{\\pageref{setup"
           << setup_count++ << "}}" << endl;
    }
  }

  setup_count = 1;
  for (vector<VariableEntry*>::iterator vit = _vars.begin();
       vit != _vars.end(); ++vit) {
    VariableEntry *ventry = *vit;

    for (vector<SetupEntry*>::iterator sit = ventry->setups.begin();
         sit != ventry->setups.end(); ++sit) {
      SetupEntry *sentry = *sit;

      for (vector<PlotEntry*>::iterator pit = sentry->plots.begin();
           pit != sentry->plots.end(); ++pit) {
        PlotEntry *pentry = *pit;

        lstr << "\\newpage" << endl;
        if (pit == sentry->plots.begin())
          lstr << "\\label{setup" << setup_count++ << "}" << endl;

        if (pentry->angle == 0.0)
          lstr << "\\includegraphics[height=\\textheight]{"
               << pentry->plot->eps_file() << "}" << endl;
        else if (pentry->angle == -90.0 || pentry->angle == 90.0)
          lstr << "\\includegraphics[height=\\textwidth,angle="
               << pentry->angle << "]{"
               << pentry->plot->eps_file() << "}" << endl;
      }
    }
  }

  lstr << "\\end{document}" << endl;
  lstr.close();

  string command = "latex " + latex_file;
  system(command.c_str());
  system(command.c_str());
  string file_head = latex_file.substr(0, latex_file.size() - 4);
  command = "dvips -Ppdf -G0 -o " + file_head + ".ps " + file_head + ".dvi";
  system(command.c_str());
  command = "ps2pdf -sPAPERSIZE=a4 ";
  command += "-dMaxSubsetPct=100 -dCompatibilityLevel=1.2 ";
  command += "-dSubsetFonts=true -dEmbedAllFonts=true ";
  command += "-dAutoFilterColorImages=false ";
  command += "-dAutoFilterGrayImages=false ";
  command += "-dColorImageFilter=/FlateEncode ";
  command += "-dGrayImageFilter=/FlateEncode ";
  command += "-dModoImageFilter=/FlateEncode ";
  command += file_head + ".ps";
  system(command.c_str());
  rename((file_head + ".pdf").c_str(), _outfile.c_str());
  unlink((file_head + ".aux").c_str());
  unlink((file_head + ".log").c_str());
  unlink((file_head + ".dvi").c_str());
  unlink((file_head + ".ps").c_str());
}


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
