//----------------------------------------------------------------------
//
//  FILE:   TempBenchmarkPlot.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Class for MOTIF benchmark seasonal plot generation.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <cstdio>

using namespace std;


// Local headers.

#include "TempBenchmarkPlot.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

TempBenchmarkPlot::TempBenchmarkPlot
(string model, const PlotSetup &setup, Component comp) :
  BenchmarkPlot(model, ATMOS, "tas", "2m Air Temperature", SEASONAL, setup),
  _component(comp)
{
  if (data_file() != "") {
    if (_component == ZONAL_TROP_LAND || _component == ZONAL_TROP_SEA) {
      char buff[256];
      sprintf(buff, "ncra -d time,0,,1 %s ###", data_file().c_str());
      _plot_data_file = TempFile::get("nc", buff);
      if (_component == ZONAL_TROP_LAND || _component == ZONAL_TROP_SEA) {
        // Zonally average from 20W to 30E for each latitude.

        string tmp_file = _plot_data_file;
        sprintf(buff, "ncks -d lat,0.0,30.0 -d lon,-20.0,30.0 %s ###",
                tmp_file.c_str());
        tmp_file = TempFile::get("nc", buff);
        sprintf(buff, "ncwa -a lon %s ###", tmp_file.c_str());
        _plot_data_file = TempFile::get("nc", buff);
      }
    } else if (_component == DJF || _component == JJA) {
      char buff[256];
      int start, end;
      switch (_component) {
      case DJF: start = 11; end = 1; break;
      case JJA: start = 5;  end = 7; break;
      }
      sprintf(buff, "ncks -d time,%d,%d %s ###",
              start, end, data_file().c_str());
      string tmp_file = TempFile::get("nc", buff);
      sprintf(buff, "ncra -d time,0,2 %s ###", tmp_file.c_str());
      _plot_data_file = TempFile::get("nc", buff);
    }
  } else
    _plot_data_file = "";
}


void TempBenchmarkPlot::plot(void)
{
  if (_plot_data_file == "") return;
  if (_component != ZONAL_TROP_LAND && _component != ZONAL_TROP_SEA) {
    map_plot(_plot_data_file);
    set_valid();
  } else
    zonally_averaged_plot(_plot_data_file);
}


void TempBenchmarkPlot::colours(FILE *fp, int &grid_colour)
{
  grid_colour = 254;
  fprintf(fp, "gsn_define_colormap(wks, \"BlueRedGray\")\n");

  if (_setup.single()) {
    fprintf(fp, "levels = fspan(-80, 50, 14)\n");
    fprintf(fp, "fill_colours = "
            "(/ 2, 17, 32, 47, 62, 77, 92, 107, 127, "
            "128, 153, 178, 203, 228, 253 /)\n");
  } else {
    fprintf(fp, "levels = (/ -20,-15,-10,-5,-2,-1,-0.1,"
            "0.1,1,2,5,10,15,20 /)\n");
    fprintf(fp, "fill_colours = "
            "(/ 2, 23, 44, 64, 85, 106, 127, 0, "
            "128, 149, 170, 190, 211, 232, 253 /)\n");
  }
}


void TempBenchmarkPlot::scaling
(float &scale_factor, float &offset, string &units)
{
  // Input data is in Kelvin; plots should be in deg. C.
  scale_factor = 1.0;
  offset = 0.0;
  if (_setup.single()) offset = -273.16;
  units = ":S:o:N:C";
}


string TempBenchmarkPlot::title_string(void)
{
  char buff[256];
  if (_component == ZONAL_TROP_LAND)
    sprintf(buff, "%s Tropical Zonal Average %s [Land] (%s)",
            _model.c_str(), _var_full_name.c_str(), string(_setup).c_str());
  else if (_component == ZONAL_TROP_SEA)
    sprintf(buff, "%s Tropical Zonal Average %s [Sea] (%s)",
            _model.c_str(), _var_full_name.c_str(), string(_setup).c_str());
  else {
    string season_string;
    switch(_component) {
    case ANNUAL: season_string = "Annual"; break;
    case DJF: season_string = "DJF"; break;
    case JJA: season_string = "JJA"; break;
    }

    sprintf(buff, "%s %s %s (%s)",
            _model.c_str(), season_string.c_str(), _var_full_name.c_str(),
            string(_setup).c_str());
  }
   
  return buff;
}


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
