//----------------------------------------------------------------------
//
//  FILE:   motif-bench.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Main program for MOTIF benchmarking plot document generator.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <iostream>
#include <cstdlib>
#include <string>

using namespace std;


// Local headers.

#include "PlotSetup.hh"
#include "SeasonalBenchmarkPlot.hh"
#include "PlotCollator.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MAIN PROGRAM
//

int main(int argc, char *argv[])
{
  // There should just be one command line argument - the model to
  // deal with.

  if (argc != 2) {
    cerr << "Usage: motif-bench <model>" << endl;
    exit(1);
  }
  string model = argv[1];

  try {
    // Set up variable and plot configuration lists.

    vector<string> vars, fvars;
    vars.push_back("tas");
    fvars.push_back("Surface Air Temperature");
    vars.push_back("pr");
    fvars.push_back("Total Precipitation");
    vars.push_back("psl");
    fvars.push_back("Sea Level Pressure");
    vars.push_back("uas");
    fvars.push_back("Surface Zonal Wind");
    vars.push_back("vas");
    fvars.push_back("Surface Meridional Wind");
    vars.push_back("wap");
    fvars.push_back("500 mb Vertical Pressure Velocity");

    vector<PlotSetup> setups;
    setups.push_back(PlotSetup(OA_0K));
    setups.push_back(PlotSetup(OAV_0K));
    setups.push_back(PlotSetup(OA_6K));
    setups.push_back(PlotSetup(OAV_6K));
    setups.push_back(PlotSetup(OA_21K));
    setups.push_back(PlotSetup(OAV_21K));
    setups.push_back(PlotSetup(OAV_0K, OA_0K));
    setups.push_back(PlotSetup(OA_6K, OA_0K));
    setups.push_back(PlotSetup(OA_21K, OA_0K));
    setups.push_back(PlotSetup(OAV_6K, OA_0K));
    setups.push_back(PlotSetup(OAV_21K, OA_0K));


    // Set up plot generation objects.

    vector<BenchmarkPlot *> plots;
    vector<string>::iterator fvit = fvars.begin();
    for (vector<string>::iterator vit = vars.begin();
         vit != vars.end(); ++vit, ++fvit) {
      string var_name = *vit, var_full_name = *fvit;

      for (vector<PlotSetup>::iterator sit = setups.begin();
           sit != setups.end(); ++sit) {
        PlotSetup setup = *sit;

        plots.push_back(new SeasonalBenchmarkPlot(model, ATMOS,
                                                  var_name, var_full_name,
                                                  setup,
                                                  SeasonalBenchmarkPlot::JJA));
        plots.push_back(new SeasonalBenchmarkPlot(model, ATMOS,
                                                  var_name, var_full_name,
                                                  setup,
                                                  SeasonalBenchmarkPlot::DJF));
        plots.push_back(new SeasonalBenchmarkPlot(model, ATMOS,
                                                  var_name, var_full_name,
                                                  setup,
                                                  SeasonalBenchmarkPlot::SON));
        plots.push_back(new SeasonalBenchmarkPlot(model, ATMOS,
                                                  var_name, var_full_name,
                                                  setup,
                                                  SeasonalBenchmarkPlot::MAM));
      }
    }


    // Generate plots.

    for (vector<BenchmarkPlot *>::iterator pit = plots.begin();
         pit != plots.end(); ++pit)
      (*pit)->plot();


    // Collate plots.

    string last_var;
    PlotSetup *last_setup;
    PlotCollator collate(string("benchmark-") + model + ".pdf",
                         model, "Seasonal Benchmarking Plots");
    vector<BenchmarkPlot *>::iterator pit = plots.begin();
    for (vector<string>::iterator vit = fvars.begin();
         vit != fvars.end(); ++vit) {
      for (vector<PlotSetup>::iterator sit = setups.begin();
           sit != setups.end(); ++sit) {

        for (int season = 0; season < 4; ++season) {
          BenchmarkPlot *plot = *pit;
          if (plot->valid()) {
            if (*vit != last_var) collate.new_variable(*vit);
            if (*sit != *last_setup) collate.new_setup(*sit);
            last_var = *vit;
            last_setup = &(*sit);
            collate.add_plot(plot);
          }
          ++pit;
        }
      }
    }
    collate();


    for (vector<BenchmarkPlot *>::iterator it = plots.begin();
         it != plots.end(); ++it)
      delete *it;
    plots.clear();
    TempFile::clean();
  } catch (const char *msg) {
    cout << "EXCEPTION: " << msg << endl;
  }
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

